var gPrioritySwitcher;

function initPrioritySwitcher()
{
	if (window.opener) {
		gPrioritySwitcher = new PrioritySwitcher(window.opener);
	} else {
		gPrioritySwitcher = new PrioritySwitcher(null);
	}
	var elems = ["cmd_delete",
							 "cmd_shiftDelete",
							 //"button-delete", //TB2 button
							 "button-mark-deleted" //TB3 button
							 ];
	for (var i=0; i<elems.length; i++) {
		var elem = document.getElementById(elems[i]);
		if (!elem) continue;
		var oncmd = elem.getAttribute("oncommand");
		if (oncmd.charAt(oncmd.length-1) != ";") oncmd = oncmd + ";";
		var str = "if (gPrioritySwitcher.filterSelection()) { " + oncmd + "}";
		elem.setAttribute("oncommand",str);
	}
}
window.addEventListener("load", initPrioritySwitcher, false);

//value of each priorities
var PSWMsgPriorityValue = {
  commit:  -2,
  reset:  -1,
  notSet:  0,
  none:    1,
  lowest:  2,
  low:     3,
  normal:  4,
  high:    5,
  highest: 6,

  convertToXPriorityValue: function(priority) {
	  var ret = null;
	  if (this.lowest <= priority && priority <= this.highest)
		ret = Math.abs(priority - 7);
	  return ret;
  },
	
  increment: function(priority) {
	  var ret = priority + 1;
	  if (ret > this.highest) ret = null;
	  return ret;
  },
  decrement: function(priority) {
	  var ret = priority - 1;
	  if (ret < this.none) ret = null;
	  return ret;
  },
  upCycle:   function(priority) {
	  var ret = this.increment(priority);
	  return ret ? ret : this.none;
  },
  downCycle: function(priority) {
	  var ret = this.decrement(priority);
	  return ret ? ret : this.highest;
  }
};

//class PrioritySwitcher
function PrioritySwitcher(parentWindow)
{
	this.parentWindow = parentWindow;
	var me = this;
	if (this.parentWindow) {
		this.pswThread = null;
		this.setDisplayedPriorityToToolbar();
	} else {
		this.pswThread = new PSWThread(this);
		var tabmail = document.getElementById("tabmail");
		var monitor = {
			onTabTitleChanged: function(aTab){},
			onTabSwitched: function(aTab, aOldTab){
				me.setDisplayedPriorityToToolbar();
			}
		};
	  tabmail.registerTabMonitor(monitor);
	}
	
	var orgFunc = FolderDisplayWidget.prototype.displayMessageChanged;
	var appendedFunc = this.setDisplayedPriorityToToolbar;

	//nsMsgDBViewCommandUpdater.prototype.displayMessageChanged =
	FolderDisplayWidget.prototype.displayMessageChanged =
	  function(aFolder, aSubject, aKeywords) {
	  	orgFunc.apply(this, arguments);
		  appendedFunc.apply(me);
	  };
	this.promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
	  getService(Components.interfaces.nsIPromptService);
	this.pswPref = new PSWPref();
	this.localeBundle = document.getElementById("PSWBundle");
	this.filterDisabled = false;
	this.switchIconTheme();
}

PrioritySwitcher.prototype.getCurrentTabMode = function() {
	var type = null;
	//if (tabmail && tabmail.tabContainer.selectedItem) {
	var tabmail = document.getElementById("tabmail");
	if (tabmail && tabmail.tabContainer.selectedIndex >= 0) {
		var tab = tabmail.tabInfo[tabmail.tabContainer.selectedIndex];
		type = tab.mode.type;
		//if (type == "glodaSearch" && (tab.query || tab.searcher || tab.collection)) {
		if (type == "glodaSearch" && tab.collection) { //distinguish gloda search result
			type = "glodaSearch-result";				
		}
	}

	return type;
}

PrioritySwitcher.prototype.switchIconTheme = function()
{
	var look = this.pswPref.getIconTheme();
	var elem = document.getElementById("prioswMenu-button");
	if (elem &&look) {
		document.getElementById("prioswMenu-button").setAttribute("look", look);
	}
}

PrioritySwitcher.prototype.changePriorityForSelMsgs = function(priority)
{
	var indices = [];
	var dbView = this.parentWindow ? this.parentWindow.gDBView : gDBView;
	try {
		indices = dbView.getIndicesForSelection({});
		var committer = null;
		if (priority == PSWMsgPriorityValue.commit) {
			committer = new PSWCommitter();
		} else if (this.pswPref.useAutoCommit()) {			
			if (dbView.msgFolder.server.type == "imap" || !this.pswPref.commitOnlyForImap()) {
				committer = new PSWCommitter();
			}
		}
		
		for (var i=0; i<indices.length; i++) {
			var index = indices[i];
			var msgKey = dbView.getKeyAt(index);
			var folder = gDBView.getFolderForViewIndex(index);
			var msgHdr = folder.GetMessageHeader(msgKey);
		
			if (priority == PSWMsgPriorityValue.reset) {
				this.resetPriorityForHdr(msgHdr, index);
			} else if (priority != PSWMsgPriorityValue.commit) {
				this.changePriorityForHdr(msgHdr, priority, index);
			}
			if (committer) committer.append(msgHdr);
		}

		if (committer) committer.commit();
	} catch(e) {
		dump(e+"\n");
	}
	
	this.setDisplayedPriorityToToolbar();
}

PrioritySwitcher.prototype.resetPriorityForHdr = function(msgHdr, rowIndex)
{
	var priorityString = "";
	var dbView = this.parentWindow ? this.parentWindow.gDBView : gDBView;
	var IOService = Components.classes['@mozilla.org/network/io-service;1'].
	  getService(Components.interfaces.nsIIOService);
	try {
		var uri = dbView.getURIForViewIndex(rowIndex);
		var messageService = messenger.messageServiceFromURI(uri);
		var messageStream = Components.classes["@mozilla.org/network/sync-stream-listener;1"].
		  createInstance().QueryInterface(Components.interfaces.nsIInputStream);
		var inputStream = Components.classes["@mozilla.org/scriptableinputstream;1"].
		  createInstance().QueryInterface(Components.interfaces.nsIScriptableInputStream);
		inputStream.init(messageStream);
		var dirService = Components.classes["@mozilla.org/file/directory_service;1"].
		  getService(Components.interfaces.nsIProperties);
		var tempDir = dirService.get("TmpD", Components.interfaces.nsIFile);

		messageService.streamMessage(uri, messageStream, msgWindow, null, false, null);
		var segment = "";
		var lines = [];
		var prevSegment = "";
		var delimiter = null;
		//remember the last priority in the header
		while (inputStream.available()) {
			segment = lines.pop() + inputStream.read(512);
			if (!delimiter) {
				var candidates = ["\r\n", "\n\r", "\n", "\r"];
				for (var i=0; i<candidates.length; i++) {
					lines = segment.split(candidates[i]);
					if (lines.length > 1) {
						delimiter = candidates[i];
						break;
					}
				}
			} else {
				lines = segment.split(delimiter);
			}

			var line = lines[0];
			while (lines.length > 1) {
				line = lines.shift();
				if  (line.match(/X-PSW-Org:\s*([a-z0-9]+)/i)) {
					if (RegExp.$1 == "none") priorityString = "";
					else priorityString = "X-Priority: " + RegExp.$1;
					break;
				} else if (line.search(/X-Priority:.*[a-z0-9]+/i) != -1) {
					priorityString = line;
				} else if (line == "") { //end of header
					break;
				}
			}
			if (line == "") break;
		}

		messageStream.close();
		inputStream.close();
	} catch (e) {
		dump(e+"\n");
		return;
	}

	if (priorityString == "") {
		this.changePriorityForHdr(msgHdr, PSWMsgPriorityValue.none, rowIndex);
	} else {
		msgHdr.setPriorityString(priorityString);
		var updater = new PSWThreadUpdater(this.parentWindow);
		updater.updateRow(rowIndex);
	}
}

PrioritySwitcher.prototype.changePriorityForHdr = function(msgHdr, priority, rowIndex)
{
	msgHdr.priority = priority;
	msgHdr.folder.updateFolder(msgWindow);
	var updater = new PSWThreadUpdater(this.parentWindow);
	updater.updateRow(rowIndex);
}

PrioritySwitcher.prototype.setDisplayedPriorityToToolbar = function()
{
	var button = document.getElementById("priosw-button");
	if (!button) return;

	var dbView = this.parentWindow ? this.parentWindow.gDBView : gDBView;
	var priority = null;
	var disabled = "false";
	try {
		var msgHdr = null;
		if (this.getCurrentTabMode() == "glodaSearch-result") throw "glodaSearch-result";
		if (this.parentWindow) {
			if (!gMessageDisplay || !gMessageDisplay.displayedMessage) {
				var me = this;
				var func = function() {
					me.setDisplayedPriorityToToolbar();
				}
				setTimeout(func);
				return;
			}
			msgHdr = gMessageDisplay.displayedMessage;
		} else {
			msgHdr = dbView.hdrForFirstSelectedMessage;
		}
		priority = msgHdr.getUint32Property("priority");
	} catch(e) {
		priority = PSWMsgPriorityValue.none;
		disabled = "true";
	}

	if (priority == PSWMsgPriorityValue.notSet) priority = PSWMsgPriorityValue.none;
	var selectedIndex = Math.abs(priority - PSWMsgPriorityValue.highest);
	button.setAttribute("disabled",disabled);
	var label = document.getElementById("prioswLabel-button");
	label.setAttribute("disabled",disabled);
	var menuItem = document.getElementById("prioswMenu-button");
	menuItem.setAttribute("disabled",disabled);
	menuItem.selectedIndex = selectedIndex;
	menuItem.setAttribute("value",priority.toString());
}

PrioritySwitcher.prototype.initToolbarPopup = function()
{
	var button = document.getElementById("priosw-button");
	if (!button) return;

	var dbView = this.parentWindow ? this.parentWindow.gDBView : gDBView;
	var resetable = new Object();
	resetable.value = false;
	var checkStatus = new Object();
	try {
		dbView.getCommandStatus(nsMsgViewCommandType.cmdRequiringMsgBody,
								 resetable, checkStatus);
	} catch(e) {
	}

	if (resetable.value) {
		document.getElementById("prioswMenu-reset").removeAttribute("disabled");
		document.getElementById("prioswMenu-commit").removeAttribute("disabled");
	} else {
		document.getElementById("prioswMenu-reset").setAttribute("disabled","true");
		document.getElementById("prioswMenu-commit").setAttribute("disabled","true");
	}
	/*
	var hostName = gFolderDisplay.selectedMessage.folder.hostname;
	if (hostName.indexOf("imap.gmail.com") != -1 || hostName.indexOf("imap.googlemail.com") != -1) {
		document.getElementById("prioswMenu-commit").setAttribute("disabled","true");
	}
	*/
}

PrioritySwitcher.prototype.initCheckedPriority = function(prefix)
{
	var priority = null;
	var resetable = new Object();
	var checkStatus = new Object();
	var dbView = this.parentWindow ? this.parentWindow.gDBView : gDBView;
	
	try {
		var msgHdr = null;
		if (this.parentWindow) {
			msgHdr = messenger.msgHdrFromURI(gCurrentMessageUri);
		} else {
			msgHdr = dbView.hdrForFirstSelectedMessage;
		}
		priority = msgHdr.getUint32Property("priority");
		resetable.value = false;
		dbView.getCommandStatus(nsMsgViewCommandType.cmdRequiringMsgBody,
								 resetable, checkStatus);
	} catch(e) {
		priority = null;
	}

	if (priority == PSWMsgPriorityValue.notSet) priority = PSWMsgPriorityValue.none;
	if (resetable.value && priority) {
		document.getElementById(prefix+"-prioswMenuItem-1").removeAttribute("disabled");
		document.getElementById(prefix+"-prioswMenuItem-2").removeAttribute("disabled");
	} else {
		document.getElementById(prefix+"-prioswMenuItem-1").setAttribute("disabled","true");
		document.getElementById(prefix+"-prioswMenuItem-2").setAttribute("disabled","true");
	}

	/*
	var hostName = gFolderDisplay.selectedMessage.folder.hostname;
	if (hostName.indexOf("imap.gmail.com")!= -1 || hostName.indexOf("imap.googlemail.com")!= -1) {
		document.getElementById(prefix+"-prioswMenuItem-2").setAttribute("disabled","true");
	}
	*/
	
	for (var i=PSWMsgPriorityValue.none; i; i=PSWMsgPriorityValue.increment(i)) {
		var menu = document.getElementById(prefix+"-prioswMenuItem"+i);
		if (menu) {
			if (!priority) {
				menu.setAttribute("disabled","true");
				menu.setAttribute("checked","false");
			} else {
				menu.removeAttribute("disabled");
				if (i == priority) menu.setAttribute("checked","true");
				else menu.setAttribute("checked","false");
			}
		}
	}
}

PrioritySwitcher.prototype.disableFilterSelection = function(disable)
{
	this.filterDisabled = disable;
}

PrioritySwitcher.prototype.filterSelection = function()
{
	if (this.filterDisabled) return true;
	
	var warnPriority = this.pswPref.getPriorityNeedsWarning();
	var warnStar = this.pswPref.warnDeletionWithStar();
	if (warnPriority == 7 && !warnStar) return true; //not confirm
	
	var ret = false;
	var del = new Array(0,false);
	var dbView = this.parentWindow ? this.parentWindow.gDBView : gDBView;
	var hdrs = [];
	if (this.parentWindow) {
		hdrs.push(gMessageDisplay.displayedMessage);
	} else { 
		hdrs = gFolderDisplay.selectedMessages;
	}
	if (dbView) {
/*
		var indices = {};
		//var count = new Object();
		if (this.parentWindow) {
			var msgKey = extractMsgKeyFromURI();
			var index = dbView.findIndexFromKey(msgKey, true);
			//indices.value = [index];
			//count.value = 1;
			indices = [index];
		} else {
			//dbView.getIndicesForSelection(indices, count); //TB2
			indices = GetSelectedIndices(gDBView);
			//dbView.getIndicesForSelection(indices);
		}
*/
		var treeView = dbView.QueryInterface(Components.interfaces.nsITreeView);
		var treeSelection = treeView.selection;
		treeSelection.clearSelection();

		dbView.QueryInterface(Components.interfaces.nsIMsgDBView);
		//for (var i = 0; i < count.value; i++) {
		//for (var i = 0; i < indices.length; i++) {
		for (var i = 0; i < hdrs.length; i++) {
			//var index = indices.value[i];
			//var index = indices[i];
			//var msgKey = dbView.getKeyAt(index);
			//var folder = gDBView.getFolderForViewIndex(index);
			//var msgHdr = folder.GetMessageHeader(msgKey);
			var msgHdr = hdrs[i];
			var msgKey = msgHdr.messageKey;
			var index = dbView.findIndexFromKey(msgKey, true);
//			var msgHdr = dbView.db.GetMsgHdrForKey(msgKey);
			if ((msgHdr.getUint32Property("priority") < warnPriority) 
				&& (!warnStar || !msgHdr.isFlagged)) {
				treeSelection.rangedSelect(index, index, true);
				if (!ret) ret = true;
			} else { //confirmation for higher priority or starred messages
				var junkScore = msgHdr.getStringProperty("junkscore"); 
				var isJunk = ((junkScore != "") && (junkScore != "0"));
				//don't confirm if the message is junk
				if (isJunk && this.pswPref.doesIgnoreHighPriorityJunkDeletion()){
					treeSelection.rangedSelect(index, index, true);
					if (!ret) ret = true;
					continue;
				}
				
				if (del[1] == false) {
				  del = this.confirmDeletion(msgHdr.mime2DecodedSubject);
				}
				
				if (del[0] == 0) {
					treeSelection.rangedSelect(index, index, true);
					if (!ret) ret = true;
				} else if (del[0] == 1) {
					ret = false;
					treeSelection.clearSelection();
					break;
				}
			}
		}
	}
	return ret;
}

PrioritySwitcher.prototype.confirmDeletion = function(subject)
{
	var title = this.localeBundle.getString("priosw.warn_del_title");
	var message1= this.localeBundle.getString("priosw.warn_del_msg2");
	var message2 = this.localeBundle.getString("priosw.warn_del_subject");
	var message3 = this.localeBundle.getString("priosw.warn_del_same");
	
	var checkValue = {value:false};
	var buttonPressed = this.promptService.confirmEx(window,
              title,
              message1+"\n"+message2+ " " + subject,
              (this.promptService.BUTTON_TITLE_YES * this.promptService.BUTTON_POS_0) +
              (this.promptService.BUTTON_TITLE_CANCEL * this.promptService.BUTTON_POS_1) +
              (this.promptService.BUTTON_TITLE_NO * this.promptService.BUTTON_POS_2) +
              this.promptService.BUTTON_POS_2_DEFAULT,
              null, null, null,
              message3,
              checkValue);

	return new Array(buttonPressed, checkValue.value);
}
